@@ -114,6 +114,41 @@ static int read_attribute(void *user_data, const char *path, harp_array data)
     return 0;
 }
 
+static int read_variable_uint8(void *user_data, const char *path, long num_elements, harp_array data)
+{
+    coda_cursor cursor;
+    long actual_num_elements;
+
+    if (coda_cursor_set_product(&cursor, ((ingest_info *)user_data)->product) != 0)
+    {
+        harp_set_error(HARP_ERROR_CODA, NULL);
+        return -1;
+    }
+    if (coda_cursor_goto(&cursor, path) != 0)
+    {
+        harp_set_error(HARP_ERROR_CODA, NULL);
+        return -1;
+    }
+    if (coda_cursor_get_num_elements(&cursor, &actual_num_elements) != 0)
+    {
+        harp_set_error(HARP_ERROR_CODA, NULL);
+        return -1;
+    }
+    if (actual_num_elements != num_elements)
+    {
+        harp_set_error(HARP_ERROR_INGESTION, "variable %s has %ld elements (expected %ld)", path, actual_num_elements,
+                       num_elements);
+        return -1;
+    }
+    if (coda_cursor_read_uint8_array(&cursor, (uint8_t *)data.int8_data, coda_array_ordering_c) != 0)
+    {
+        harp_set_error(HARP_ERROR_CODA, NULL);
+        return -1;
+    }
+
+    return 0;
+}
+
 static int read_variable_float(void *user_data, const char *path, long num_elements, harp_array data)
 {
     coda_cursor cursor;
@@ -312,11 +347,16 @@ static int read_longitude(void *user_data, harp_array data)
 }
 
 static int read_pressure(void *user_data, harp_array data)
+{
+    return read_variable_float(user_data, "PRESSURE", ((ingest_info *)user_data)->num_time, data);
+}
+
+static int read_pressure_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "PRESSURE_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
 
-static int read_pressure_uncertainty(void *user_data, harp_array data)
+static int read_pressure_insitu_uncertainty(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "PRESSURE_INSITU_UNCERTAINTY_COMBINED_STANDARD",
                                ((ingest_info *)user_data)->num_time, data);
@@ -334,64 +374,106 @@ static int read_altitude_gph_uncertainty(void *user_data, harp_array data)
 }
 
 static int read_temperature(void *user_data, harp_array data)
+{
+    return read_variable_float(user_data, "TEMPERATURE", ((ingest_info *)user_data)->num_time, data);
+}
+
+static int read_temperature_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "TEMPERATURE_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
 
-static int read_temperature_uncertainty(void *user_data, harp_array data)
+static int read_temperature_insitu_uncertainty(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "TEMPERATURE_INSITU_UNCERTAINTY_COMBINED_STANDARD",
                                ((ingest_info *)user_data)->num_time, data);
 }
 
 static int read_relative_humidity(void *user_data, harp_array data)
+{
+    return read_variable_float(user_data, "HUMIDITY_RELATIVE", ((ingest_info *)user_data)->num_time, data);
+}
+
+static int read_relative_humidity_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "HUMIDITY_RELATIVE_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
 
-static int read_relative_humidity_uncertainty(void *user_data, harp_array data)
+static int read_relative_humidity_insitu_uncertainty(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "HUMIDITY_RELATIVE_INSITU_UNCERTAINTY_COMBINED_STANDARD",
                                ((ingest_info *)user_data)->num_time, data);
 }
 
 static int read_O3_partial_pressure(void *user_data, harp_array data)
+{
+    return read_variable_float(user_data, "O3_PARTIAL_PRESSURE", ((ingest_info *)user_data)->num_time, data);
+}
+
+static int read_O3_partial_pressure_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "O3_PARTIAL_PRESSURE_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
 
 static int read_O3_partial_pressure_uncertainty(void *user_data, harp_array data)
+{
+    return read_variable_float(user_data, "O3_PARTIAL_PRESSURE_UNCERTAINTY_COMBINED_STANDARD",
+                               ((ingest_info *)user_data)->num_time, data);
+}
+
+static int read_O3_partial_pressure_insitu_uncertainty(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "O3_PARTIAL_PRESSURE_INSITU_UNCERTAINTY_COMBINED_STANDARD",
                                ((ingest_info *)user_data)->num_time, data);
 }
 
-static int read_O3_volume_mixing_ratio(void *user_data, harp_array data)
+static int read_O3_partial_pressure_flag(void *user_data, harp_array data)
+{
+    /* we read the uint8 data into a int8 array because HARP does not support unsigned integers */
+    return read_variable_uint8(user_data, "O3_PARTIAL_PRESSURE_FLAG", ((ingest_info *)user_data)->num_time, data);
+}
+
+static int read_O3_volume_mixing_ratio_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "O3_MIXING_RATIO_VOLUME_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
 
-static int read_O3_number_density(void *user_data, harp_array data)
+static int read_O3_column(void *user_data, harp_array data)
+{
+    return read_variable_float(user_data, "O3_COLUMN", 1, data);
+}
+
+static int read_O3_number_density_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "O3_NUMBER_DENSITY_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
 
 static int read_wind_speed(void *user_data, harp_array data)
+{
+    return read_variable_float(user_data, "WIND_SPEED", ((ingest_info *)user_data)->num_time, data);
+}
+
+static int read_wind_speed_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "WIND_SPEED_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
 
 static int read_wind_direction(void *user_data, harp_array data)
+{
+    return read_variable_float(user_data, "WIND_DIRECTION", ((ingest_info *)user_data)->num_time, data);
+}
+
+static int read_wind_direction_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "WIND_DIRECTION_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
 
-static int read_potential_temperature(void *user_data, harp_array data)
+static int read_potential_temperature_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "POTENTIAL_TEMPERATURE_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
 
-static int read_h2o_volume_mixing_ratio(void *user_data, harp_array data)
+static int read_h2o_volume_mixing_ratio_insitu(void *user_data, harp_array data)
 {
     return read_variable_float(user_data, "H2O_MIXING_RATIO_VOLUME_INSITU", ((ingest_info *)user_data)->num_time, data);
 }
@@ -430,7 +512,7 @@ static int get_product_definition(const harp_ingestion_module *module, coda_prod
                                   harp_product_definition **definition)
 {
     coda_cursor cursor;
-    char template_name[18];
+    char template_name[25];
     long length;
     long i;
 
@@ -444,29 +526,21 @@ static int get_product_definition(const harp_ingestion_module *module, coda_prod
         harp_set_error(HARP_ERROR_UNSUPPORTED_PRODUCT, "could not find DATA_TEMPLATE global attribute");
         return -1;
     }
-    if (coda_cursor_get_string_length(&cursor, &length) != 0)
-    {
-        harp_set_error(HARP_ERROR_CODA, NULL);
-        return -1;
-    }
-    /* template should match the pattern "GEOMS-TE-SONDE-xxx" */
-    if (length < 18)
-    {
-        harp_set_error(HARP_ERROR_UNSUPPORTED_PRODUCT, "invalid string length for DATA_TEMPLATE global attribute");
-        return -1;
-    }
-    if (coda_cursor_read_string(&cursor, template_name, 19) != 0)
+    /* template should match the pattern "GEOMS-TE-SONDE[-O3]-xxx" */
+    if (coda_cursor_read_string(&cursor, template_name, 25) != 0)
     {
         harp_set_error(HARP_ERROR_UNSUPPORTED_PRODUCT, NULL);
         return -1;
     }
 
+    length = strlen(template_name);
     for (i = 0; i < module->num_product_definitions; i++)
     {
         /* match against product definition name: '<template_name>-<profile|points>' */
-        if (strncmp(template_name, module->product_definition[i]->name, 18) == 0)
+        if (strncmp(template_name, module->product_definition[i]->name, length) == 0)
         {
-            if (strcmp(&module->product_definition[i]->name[19], convert_to_profile ? "profile" : "points") == 0)
+            if (strcmp(&module->product_definition[i]->name[length + 1], convert_to_profile ? "profile" : "points") ==
+                0)
             {
                 *definition = module->product_definition[i];
                 return 0;
@@ -515,7 +589,7 @@ static int get_dimensions(ingest_info *info)
     return 0;
 }
 
-static int get_optional_variable_availability(ingest_info *info)
+static int get_optional_variable_availability_v2(ingest_info *info)
 {
     coda_cursor cursor;
 
@@ -548,7 +622,7 @@ static int get_optional_variable_availability(ingest_info *info)
     return 0;
 }
 
-static int get_latlon_length(ingest_info *info)
+static int get_latlon_length_v2(ingest_info *info)
 {
     coda_cursor cursor;
     long num_elements;
@@ -601,13 +675,12 @@ static int ingestion_init(const harp_ingestion_module *module, coda_product *pro
         return -1;
     }
     info->definition = *definition;
-    info->product_version = info->definition->name[16] - '0';
-    info->has_scalar_latlon = 0;
-    info->has_o3 = 0;
-    info->has_wind_speed = 0;
-    info->has_wind_direction = 0;
-    info->has_potential_temperature = 0;
-    info->has_h2o = 0;
+    if (coda_get_product_version(info->product, &info->product_version) != 0)
+    {
+        harp_set_error(HARP_ERROR_CODA, NULL);
+        return -1;
+    }
+    info->product_version = info->product_version % 100;        /* strip the leading netcdf/hdf indicator */
 
     if (get_dimensions(info) != 0)
     {
@@ -615,23 +688,42 @@ static int ingestion_init(const harp_ingestion_module *module, coda_product *pro
         return -1;
     }
 
-    if (get_optional_variable_availability(info) != 0)
+    if (info->product_version < 3)
     {
-        ingestion_done(info);
-        return -1;
-    }
+        info->has_scalar_latlon = 0;
+        info->has_o3 = 0;
+        info->has_wind_speed = 0;
+        info->has_wind_direction = 0;
+        info->has_potential_temperature = 0;
+        info->has_h2o = 0;
 
-    if (get_latlon_length(info) != 0)
+        if (get_optional_variable_availability_v2(info) != 0)
+        {
+            ingestion_done(info);
+            return -1;
+        }
+
+        if (get_latlon_length_v2(info) != 0)
+        {
+            ingestion_done(info);
+            return -1;
+        }
+    }
+    else
     {
-        ingestion_done(info);
-        return -1;
+        info->has_scalar_latlon = 0;
+        info->has_o3 = 1;
+        info->has_wind_speed = 1;
+        info->has_wind_direction = 1;
+        info->has_potential_temperature = 0;
+        info->has_h2o = 0;
     }
 
     *user_data = info;
     return 0;
 }
 
-static int init_product_definition(harp_ingestion_module *module, int convert_to_profile, int version)
+static int init_product_definition_v2(harp_ingestion_module *module, int convert_to_profile)
 {
     harp_variable_definition *variable_definition;
     harp_product_definition *product_definition;
@@ -640,9 +732,8 @@ static int init_product_definition(harp_ingestion_module *module, int convert_to
     char product_description[MAX_DESCRIPTION_LENGTH];
     const char *description;
 
-    snprintf(product_name, MAX_NAME_LENGTH, "GEOMS-TE-SONDE-%03d-%s", version,
-             convert_to_profile ? "profile" : "points");
-    snprintf(product_description, MAX_DESCRIPTION_LENGTH, "GEOMS template for Sonde v%03d (%s)", version,
+    snprintf(product_name, MAX_NAME_LENGTH, "GEOMS-TE-SONDE-002-%s", convert_to_profile ? "profile" : "points");
+    snprintf(product_description, MAX_DESCRIPTION_LENGTH, "GEOMS template for Sonde v002 (%s)",
              convert_to_profile ? "as single profile" : "as timeseries of points");
     product_definition = harp_ingestion_register_product(module, product_name, product_description, read_dimensions);
     harp_product_definition_add_mapping(product_definition, NULL,
@@ -667,16 +758,18 @@ static int init_product_definition(harp_ingestion_module *module, int convert_to
 
         /* datetime_start */
         description = "time of first measurement of the profile";
-        variable_definition = harp_ingestion_register_variable_full_read
-            (product_definition, "datetime_start", harp_type_double, 1, dimension_type, NULL, description,
-             "days since 2000-01-01", NULL, read_datetime_start);
+        variable_definition =
+            harp_ingestion_register_variable_full_read(product_definition, "datetime_start", harp_type_double, 1,
+                                                       dimension_type, NULL, description, "days since 2000-01-01", NULL,
+                                                       read_datetime_start);
         harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/DATETIME[0]", NULL);
 
         /* datetime_stop */
         description = "time of last measurement of the profile";
-        variable_definition = harp_ingestion_register_variable_full_read
-            (product_definition, "datetime_stop", harp_type_double, 1, dimension_type, NULL, description,
-             "days since 2000-01-01", NULL, read_datetime_stop);
+        variable_definition =
+            harp_ingestion_register_variable_full_read(product_definition, "datetime_stop", harp_type_double, 1,
+                                                       dimension_type, NULL, description, "days since 2000-01-01", NULL,
+                                                       read_datetime_stop);
         harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/DATETIME[N-1]", NULL);
 
         dimension_type[0] = harp_dimension_vertical;
@@ -687,148 +780,321 @@ static int init_product_definition(harp_ingestion_module *module, int convert_to
 
         /* datetime */
         description = "time of the measurement";
-        variable_definition = harp_ingestion_register_variable_full_read
-            (product_definition, "datetime", harp_type_double, 1, dimension_type, NULL, description,
-             "days since 2000-01-01", NULL, read_datetime);
+        variable_definition =
+            harp_ingestion_register_variable_full_read(product_definition, "datetime", harp_type_double, 1,
+                                                       dimension_type, NULL, description, "days since 2000-01-01", NULL,
+                                                       read_datetime);
         harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/DATETIME", NULL);
     }
 
     /* latitude */
     description = "latitude";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "latitude", harp_type_float, 1, dimension_type, NULL, description, "degree_north", NULL,
-         read_latitude);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "latitude", harp_type_float, 1, dimension_type,
+                                                   NULL, description, "degree_north", NULL, read_latitude);
     description = "if the latitude is a scalar it is replicated for each profile point";
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/LATITUDE", description);
 
     /* longitude */
     description = "longitude";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "longitude", harp_type_float, 1, dimension_type, NULL, description, "degree_east", NULL,
-         read_longitude);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "longitude", harp_type_float, 1, dimension_type,
+                                                   NULL, description, "degree_east", NULL, read_longitude);
     description = "if the longitude is a scalar it is replicated for each profile point";
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/LONGITUDE", description);
 
     /* pressure */
     description = "pressure measurement from PTU sonde";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "pressure", harp_type_float, 1, dimension_type, NULL, description, "hPa", NULL,
-         read_pressure);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "pressure", harp_type_float, 1, dimension_type,
+                                                   NULL, description, "hPa", NULL, read_pressure_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/PRESSURE_INSITU", NULL);
 
     /* pressure_uncertainty */
     description = "1 sigma uncertainty estimate of the pressure measurement";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "pressure_uncertainty", harp_type_float, 1, dimension_type, NULL, description, "hPa", NULL,
-         read_pressure_uncertainty);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "pressure_uncertainty", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "hPa", NULL,
+                                                   read_pressure_insitu_uncertainty);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL,
                                          "/PRESSURE_INSITU_UNCERTAINTY.COMBINED.STANDARD", NULL);
 
     /* geopotential_height */
     description = "calculated sonde GPH";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "geopotential_height", harp_type_float, 1, dimension_type, NULL, description, "m", NULL,
-         read_altitude_gph);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "geopotential_height", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "m", NULL, read_altitude_gph);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/ALTITUDE.GPH", NULL);
 
     /* geopotential_height_uncertainty */
     description = "1 sigma uncertainty estimate of the altitude measurement";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "geopotential_height_uncertainty", harp_type_float, 1, dimension_type, NULL, description,
-         "m", NULL, read_altitude_gph_uncertainty);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "geopotential_height_uncertainty",
+                                                   harp_type_float, 1, dimension_type, NULL, description, "m", NULL,
+                                                   read_altitude_gph_uncertainty);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/ALTITUDE.GPH_UNCERTAINTY.COMBINED.STANDARD",
                                          NULL);
 
     /* temperature */
     description = "temperature measurement from PTU sonde";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "temperature", harp_type_float, 1, dimension_type, NULL, description, "K", NULL,
-         read_temperature);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "temperature", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "K", NULL,
+                                                   read_temperature_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/TEMPERATURE_INSITU", NULL);
 
     /* temperature_uncertainty */
     description = "1 sigma uncertainty estimate of the temperature measurement";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "temperature_uncertainty", harp_type_float, 1, dimension_type, NULL, description, "K",
-         NULL, read_temperature_uncertainty);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "temperature_uncertainty", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "K", NULL,
+                                                   read_temperature_insitu_uncertainty);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL,
                                          "/TEMPERATURE_INSITU_UNCERTAINTY.COMBINED.STANDARD", NULL);
 
     /* relative_humidity */
     description = "relative humidity from PTU sonde";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "relative_humidity", harp_type_float, 1, dimension_type, NULL, description, "%", NULL,
-         read_relative_humidity);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "relative_humidity", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "%", NULL,
+                                                   read_relative_humidity_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/HUMIDITY.RELATIVE_INSITU", NULL);
 
     /* relative_humidity_uncertainty */
     description = "1 sigma uncertainty estimate of the relative humidity";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "relative_humidity_uncertainty", harp_type_float, 1, dimension_type, NULL, description,
-         "%", NULL, read_relative_humidity_uncertainty);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "relative_humidity_uncertainty", harp_type_float,
+                                                   1, dimension_type, NULL, description, "%", NULL,
+                                                   read_relative_humidity_insitu_uncertainty);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL,
                                          "/HUMIDITY.RELATIVE_INSITU_UNCERTAINTY.COMBINED.STANDARD", NULL);
 
     /* O3_partial_pressure */
     description = "in situ partial pressure";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "O3_partial_pressure", harp_type_float, 1, dimension_type, NULL, description, "mPa",
-         include_o3, read_O3_partial_pressure);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "O3_partial_pressure", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "mPa", include_o3,
+                                                   read_O3_partial_pressure_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/O3.PARTIAL.PRESSURE_INSITU", NULL);
 
     /* O3_partial_pressure_uncertainty */
     description = "1 sigma uncertainty estimate of the partial pressure";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "O3_partial_pressure_uncertainty", harp_type_float, 1, dimension_type, NULL, description,
-         "mPa", include_o3, read_O3_partial_pressure_uncertainty);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "O3_partial_pressure_uncertainty",
+                                                   harp_type_float, 1, dimension_type, NULL, description, "mPa",
+                                                   include_o3, read_O3_partial_pressure_insitu_uncertainty);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL,
                                          "/O3.PARTIAL.PRESSURE_INSITU_UNCERTAINTY.COMBINED.STANDARD", NULL);
 
     /* O3_volume_mixing_ratio */
     description = "calculated in situ ozone volumetric mixing ratio from ozone sonde";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "O3_volume_mixing_ratio", harp_type_float, 1, dimension_type, NULL, description, "ppmv",
-         include_o3, read_O3_volume_mixing_ratio);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "O3_volume_mixing_ratio", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "ppmv", include_o3,
+                                                   read_O3_volume_mixing_ratio_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/O3.MIXING.RATIO.VOLUME_INSITU", NULL);
 
     /* O3_number_density */
     description = "calculated in situ ozone number density from ozone sonde";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "O3_number_density", harp_type_float, 1, dimension_type, NULL, description, "molec/m3",
-         include_o3, read_O3_number_density);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "O3_number_density", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "molec/m3", include_o3,
+                                                   read_O3_number_density_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/O3.NUMBER.DENSITY_INSITU", NULL);
 
     /* wind_speed */
     description = "wind speed from instrument package";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "wind_speed", harp_type_float, 1, dimension_type, NULL, description, "m/s",
-         include_wind_speed, read_wind_speed);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "wind_speed", harp_type_float, 1, dimension_type,
+                                                   NULL, description, "m/s", include_wind_speed,
+                                                   read_wind_speed_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/WIND.SPEED_INSITU", NULL);
 
     /* wind_direction */
     description = "wind direction from instrument package";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "wind_direction", harp_type_float, 1, dimension_type, NULL, description, "degree",
-         include_wind_direction, read_wind_direction);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "wind_direction", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "degree", include_wind_direction,
+                                                   read_wind_direction_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/WIND.DIRECTION_INSITU", NULL);
 
     /* potential_temperature */
     description = "calculated in situ potential temperature from sonde";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "potential_temperature", harp_type_float, 1, dimension_type, NULL, description, "K",
-         include_potential_temperature, read_potential_temperature);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "potential_temperature", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "K",
+                                                   include_potential_temperature, read_potential_temperature_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/POTENTIAL.TEMPERATURE_INSITU", NULL);
 
     /* h2o_volume_mixing_ratio */
     description = "calculated in situ water vapor volumetric mixing ratio from sonde";
-    variable_definition = harp_ingestion_register_variable_full_read
-        (product_definition, "h2o_volume_mixing_ratio", harp_type_float, 1, dimension_type, NULL, description, "ppmv",
-         include_h2o, read_h2o_volume_mixing_ratio);
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "h2o_volume_mixing_ratio", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "ppmv", include_h2o,
+                                                   read_h2o_volume_mixing_ratio_insitu);
     harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/H2O.MIXING.RATIO.VOLUME_INSITU", NULL);
 
     return 0;
 }
 
+static int init_product_definition_v3(harp_ingestion_module *module, int convert_to_profile)
+{
+    harp_variable_definition *variable_definition;
+    harp_product_definition *product_definition;
+    harp_dimension_type dimension_type[1];
+    char product_name[MAX_NAME_LENGTH];
+    char product_description[MAX_DESCRIPTION_LENGTH];
+    const char *description;
+
+    snprintf(product_name, MAX_NAME_LENGTH, "GEOMS-TE-SONDE-O3-003-%s", convert_to_profile ? "profile" : "points");
+    snprintf(product_description, MAX_DESCRIPTION_LENGTH, "GEOMS template for Sonde v003 (%s)",
+             convert_to_profile ? "as single profile" : "as timeseries of points");
+    product_definition = harp_ingestion_register_product(module, product_name, product_description, read_dimensions);
+    harp_product_definition_add_mapping(product_definition, NULL,
+                                        convert_to_profile ? "profile unset" : "profile=false");
+
+    /* sensor_name */
+    description = "name of the sensor";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "sensor_name", harp_type_string, 0, NULL, NULL,
+                                                   description, NULL, NULL, read_data_source);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/@DATA.SOURCE", NULL);
+
+    /* location_name */
+    description = "name of the site at which the sensor is located";
+    variable_definition = harp_ingestion_register_variable_full_read(product_definition, "location_name",
+                                                                     harp_type_string, 0, NULL, NULL, description, NULL,
+                                                                     NULL, read_data_location);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/@DATA.LOCATION", NULL);
+
+    if (convert_to_profile)
+    {
+        dimension_type[0] = harp_dimension_time;
+
+        /* datetime_start */
+        description = "time of first measurement of the profile";
+        variable_definition =
+            harp_ingestion_register_variable_full_read(product_definition, "datetime_start", harp_type_double, 1,
+                                                       dimension_type, NULL, description, "days since 2000-01-01", NULL,
+                                                       read_datetime_start);
+        harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/DATETIME[0]", NULL);
+
+        /* datetime_stop */
+        description = "time of last measurement of the profile";
+        variable_definition =
+            harp_ingestion_register_variable_full_read(product_definition, "datetime_stop", harp_type_double, 1,
+                                                       dimension_type, NULL, description, "days since 2000-01-01", NULL,
+                                                       read_datetime_stop);
+        harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/DATETIME[N-1]", NULL);
+
+        dimension_type[0] = harp_dimension_vertical;
+    }
+    else
+    {
+        dimension_type[0] = harp_dimension_time;
+
+        /* datetime */
+        description = "time of the measurement";
+        variable_definition =
+            harp_ingestion_register_variable_full_read(product_definition, "datetime", harp_type_double, 1,
+                                                       dimension_type, NULL, description, "days since 2000-01-01", NULL,
+                                                       read_datetime);
+        harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/DATETIME", NULL);
+    }
+
+    /* latitude */
+    description = "latitude";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "latitude", harp_type_float, 1, dimension_type,
+                                                   NULL, description, "degree_north", NULL, read_latitude);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/LATITUDE", description);
+
+    /* longitude */
+    description = "longitude";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "longitude", harp_type_float, 1, dimension_type,
+                                                   NULL, description, "degree_east", NULL, read_longitude);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/LONGITUDE", description);
+
+    /* pressure */
+    description = "pressure measurement from PTU sonde";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "pressure", harp_type_float, 1, dimension_type,
+                                                   NULL, description, "hPa", NULL, read_pressure);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/PRESSURE", NULL);
+
+    /* geopotential_height */
+    description = "Geopotential height above mean sea level";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "geopotential_height", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "m", NULL, read_altitude_gph);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/ALTITUDE.GPH", NULL);
+
+    /* temperature */
+    description = "temperature measurement from PTU sonde";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "temperature", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "K", NULL, read_temperature);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/TEMPERATURE", NULL);
+
+    /* relative_humidity */
+    description = "relative humidity from PTU sonde";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "relative_humidity", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "%", NULL,
+                                                   read_relative_humidity);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/HUMIDITY.RELATIVE", NULL);
+
+    /* O3_partial_pressure */
+    description = "in situ partial pressure";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "O3_partial_pressure", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "mPa", NULL,
+                                                   read_O3_partial_pressure);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/O3.PARTIAL.PRESSURE", NULL);
+
+    /* O3_partial_pressure_uncertainty */
+    description = "1 sigma uncertainty estimate of the partial pressure";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "O3_partial_pressure_uncertainty",
+                                                   harp_type_float, 1, dimension_type, NULL, description, "mPa", NULL,
+                                                   read_O3_partial_pressure_uncertainty);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL,
+                                         "/O3.PARTIAL.PRESSURE_UNCERTAINTY.COMBINED.STANDARD", NULL);
+
+    /* O3_partial_pressure_validity */
+    description = "FlagDataReliability (using the WMO Code 0-33-020 convention)";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "O3_partial_pressure_validity",
+                                                   harp_type_int8, 1, dimension_type, NULL, description, NULL, NULL,
+                                                   read_O3_partial_pressure_flag);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/O3.PARTIAL.PRESSURE_FLAG", NULL);
+
+    /* O3_column_number_density */
+    description = "ozone column sonde";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "O3_column_number_density", harp_type_float, 0,
+                                                   NULL, NULL, description, "DU", NULL, read_O3_column);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/O3.COLUMN", NULL);
+
+    /* wind_speed */
+    description = "wind speed from instrument package";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "wind_speed", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "m/s", NULL, read_wind_speed);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/WIND.SPEED", NULL);
+
+    /* wind_direction */
+    description = "wind direction from instrument package";
+    variable_definition =
+        harp_ingestion_register_variable_full_read(product_definition, "wind_direction", harp_type_float, 1,
+                                                   dimension_type, NULL, description, "degree", NULL,
+                                                   read_wind_direction);
+    harp_variable_definition_add_mapping(variable_definition, NULL, NULL, "/WIND.DIRECTION", NULL);
+
+    return 0;
+}
+
 int harp_ingestion_module_geoms_sonde_init(void)
 {
     const char *profile_options[] = { "false" };
@@ -840,8 +1106,10 @@ int harp_ingestion_module_geoms_sonde_init(void)
     harp_ingestion_register_option(module, "profile", "whether to ingest the sonde profile as a vertical profile "
                                    "(default) or as a timeseries of points (profile=false)", 1, profile_options);
 
-    init_product_definition(module, 0, 2);
-    init_product_definition(module, 1, 2);
+    init_product_definition_v2(module, 0);
+    init_product_definition_v2(module, 1);
+    init_product_definition_v3(module, 0);
+    init_product_definition_v3(module, 1);
 
     return 0;
 }
